local mod = LibStub("AceAddon-3.0"):NewAddon("WintergraspAutoRaid", "AceEvent-3.0", "AceTimer-3.0")
local LDB = LibStub:GetLibrary("LibDataBroker-1.1")
local LSM = LibStub("LibSharedMedia-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local ACD3 = LibStub("AceConfigDialog-3.0")

local db
local words = {"inv", "invite"}
local invited = {}
local optFrame

local lastRealTime
local lastWGTime
local function secondsUntilWG()
	local t = GetWintergraspWaitTime()
	if t == lastWGTime and GetTime() - lastRealTime > 10 then
		local newTime = lastWGTime - (GetTime() - lastRealTime)
		if newTime > 0 then
			return newTime
		else
			return false
		end
	else
		lastWGTime = t
		lastRealTime = GetTime()
		return t
	end
end

local options = {
	type = "group",
	args = {
		alarm = {
			type = "toggle",
			name = L["Wintergrasp Alarm"],
			desc = L["Alert the player when Wintergrasp is about to start."],
			order = 1,
			get = function()
				return db.profile.alert
			end,
			set = function(info, v)
				db.profile.alert = v
			end
		},
		alarmTime = {
			order = 2,
			type = "range",
			name = L["Alarm Time"],
			desc = L["When to let the player know that Wintergrasp is about to start."],
			min = 5,
			max = 900,
			step = 30,
			bigStep = 30,
			get = function()
				return db.profile.alertTime
			end,
			set = function(info, v)
				db.profile.alertTime = v
			end,
			disabled = function() return not db.profile.alert end,
		},
		alarmSound = {
			order = 3,
			type = "select",
			dialogControl = 'LSM30_Sound',
			order = 8,
			name = L["Sound to play"],
			desc = L["Sound to play"],
			values = AceGUIWidgetLSMlists.sound,
			get = function()
				return db.profile.alertSound
			end,
			set = function(info, v)
				db.profile.alertSound = v
			end,
			disabled = function() return not db.profile.alert end,
		},
		inviteInDalaran = {
			order = 4,
			type = "toggle",
			name = L["Invite in Dalaran"],
			desc = L["Causes WAR to auto-invite people in Dalaran when Wintergrasp is about to start."],
			get = function()
				return db.profile.inviteInDalaran
			end,
			set = function(info, v)
				db.profile.inviteInDalaran = v
			end
		}
	}
}

local ldbObj = LDB:NewDataObject("WintergraspAutoRaid", {
	type = "data source",
	icon = [[Interface\ICONS\INV_Jewelry_Ring_66]],
	label = "WintergraspAutoRaid",
	text  = "Active",
	
	OnClick = function(self, btn)
		if btn == "RightButton" then
			InterfaceOptionsFrame_OpenToCategory(optFrame)
		else
			db.profile.active = not db.profile.active
			local dummy = db.profile.active and mod:Enable() or mod:Disable()
		end
	end,
	
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine( ("|cff00cc00%s"):format("Wintergrasp AutoRaid") )
		tooltip:AddDoubleLine( L["Automatic Invites"], ("|cffffffff%s|h"):format(db.profile.active and L["On"] or L["Off"]) )
		local t = secondsUntilWG()
		if t == false then		-- not in Northrend
			tooltip:AddLine(("|cffff0000%s|h"):format(L["Must be in Northrend to get Wintergrasp timer"]))
		else
			local h = math.floor(t / 3600)
			local m = math.floor((t - (3600 * h)) / 60)
			local s = t - (h * 3600) - (m * 60)
			tooltip:AddDoubleLine( L["Time Until Wintergrasp"], string.format("|cffffffff%02d:%02d:%02d|h", h, m, s))
			tooltip:AddLine(" ")
			tooltip:AddLine(("|c66666666%s"):format(L["Left-click to toggle auto-invites"]))
			tooltip:AddLine(("|c66666666%s"):format(L["Right-click to change settings"]))
		end
	end,
})

local function inWintergraspAndBattleIsActive()
	local waitTime = secondsUntilWG()
	local area = GetCurrentMapAreaID()
	local validArea = area == WORLDMAP_WINTERGRASP_ID or (area == 505 and db.profile.inviteInDalaran) -- 505 is Dalaran
	return validArea and not waitTime or waitTime < 300
end

local function autoInvite(frame, event, msg, name, ...)
	if inWintergraspAndBattleIsActive() then
		local t = GetTime()
		local solo = GetNumPartyMembers() == 0 and GetNumRaidMembers() == 0
		local hasRoom = GetNumRaidMembers() < 40
		local zoneID = select(5, ...)
		local validPartyStatus = IsPartyLeader() or IsRaidLeader() or IsRaidOfficer() or solo
		if not UnitIsAFK("player") and zoneID <= 2 and hasRoom and validPartyStatus then
			for _, v in ipairs(words) do
				if strlower(msg):match(v) then
					if name and t - (invited[name] or 0) > 3 then
						InviteUnit(name)
						invited[name] = t
					end
				end
			end
		end
	end
	return false, msg, name, ...
end

local defaults = {
	profile = {
		active = true,
		inviteInDalaran = false,
		alert = true,
		alertTime = 300
	}
}
function mod:OnInitialize()
	db = LibStub("AceDB-3.0"):New("WintergraspAutoRaidDB", defaults)
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("WintergraspAutoRaid", options)
	optFrame = ACD3:AddToBlizOptions("WintergraspAutoRaid")
end

function mod:OnEnable()
	ldbObj.text = ("|cff00ff00%s|h"):format(L["Active"])
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", autoInvite)
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:ScheduleRepeatingTimer("Alarm", math.min(120, db.profile.alertTime))
	if not db.profile.active then
		self:Disable()
	end
end

function mod:OnDisable()
	ldbObj.text = ("|cffff0000%s|h"):format(L["Inactive"])
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_CHANNEL", autoInvite)	
	self:UnregisterEvent("PARTY_MEMBERS_CHANGED")
end

local color = {r = 1, g = 1, b = 0.5, a = 1}
function mod:Alarm(force)
	local wgTime = secondsUntilWG()
	if wgTime and self.lastWGTime ~= wgTime and wgTime < db.profile.alertTime or force then
		local m = math.floor(wgTime / 60)
		local s = wgTime - (m * 60)
		PlaySoundFile(LSM:Fetch("sound", db.profile.alertSound))
		RaidNotice_AddMessage(RaidWarningFrame, ("Wintergrasp in %02d:%02d"):format(m, s), color)
	end
end

function mod:PARTY_MEMBERS_CHANGED()
	if inWintergraspAndBattleIsActive() then
		if IsPartyLeader() and GetNumRaidMembers() == 0 then
			ConvertToRaid()
		-- elseif IsPartyLeader() then
			-- for i = 1, GetNumRaidMembers() do
			-- 	PromoteToAssistant("raid" .. i)
			-- end
		end
	end
end
